
// Copyright in this code is held by Dr B. R. Gladman but free direct or
// derivative use is permitted subject to acknowledgement of its origin.  
// Dr. B. R. Gladman (gladman@seven77.demon.co.uk).

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cctype>
#include <string>

#include "aes_defs.h"

// Strings to locate lines in test vector files

const int	 no_lhdr = 6;
std::string  fstr[no_lhdr] = { "KEYSIZE=", "I=", "IV=", "KEY=", "PT=", "CT=" };

char	*hxx = "0123456789abcdef";

// Hexadecimal conversion from hex character to a number in range 0 <= no <= 15

inline int to_hex(int ch)
{
	return (ch & 15) + (ch >= '0' && ch <= '9' ? 0 : 9);
}

// Read from a string of hexadecimal digits into an array of 8-bit bytes
// Each pair of hexadecimal digits is used to compile one 8-bit byte and
// the first (last) digit forms the most (least) significant four bits 
// of each byte. The hexadecimal digits are read from left (low) to right
// (high) and are placed in increasing index positions in the byte array
 
int block_in(u1byte b[], std::string::iterator p, std::string::iterator top)
{   int i = 0;

    if((top - p) & 1)

        return -1;

    while(p < top)
    {
        if(isxdigit(*p) && isxdigit(*(p + 1)))
        {
            b[i++] = (to_hex(*p) << 4) + to_hex(*(p + 1)); p += 2;
        }
        else

            break;
    }

    return i;
}

// Output an array of bytes to a string of hexadecimal digits using
// the conventions described for input above.  But if type (ty) = 1
// it outputs a test number in decimal notation

void block_out(const u1byte ty, const u1byte b[], std::ostream &outf, u4byte len = 16)
{
    if(ty <= 1)

        outf << std::endl << std::endl << fstr[ty] << len;
    
    else
    {   
		outf << std::endl << fstr[ty];

        for(int i = 0; i < len; ++i)

            outf << hxx[(b[i] >> 4) & 15] << hxx[b[i] & 15];
    }
}

// clears an array of 8-bit bytes

void block_clear(u1byte l[], u4byte len = 16)
{	u1byte	*lp = l;

	while(lp < l + len)
	
		*lp++ = 0;
} 

// reverses the order of an array of 8-bit bytes

void block_reverse(u1byte b[], u4byte len = 16)
{
	u1byte	*blo = b, *bhi = b + len - 1, by;

	while(blo < bhi)
	
		by = *blo, *blo++ = *bhi, *bhi-- = by;
} 

// copies an array of 8-bit bytes

void block_copy(u1byte l[], const u1byte r[], const u4byte len = 16)
{	u1byte			*lp = l;
	const u1byte	*rp = r;

	while(lp < l + len)
	
		*lp++ = *rp++;
} 

// xors one array of 8-bit bytes with another

void block_xor(u1byte l[], const u1byte r[], const u4byte len = 16)
{	u1byte			*lp = l;
	const u1byte	*rp = r;

	while(lp < l + len)
	
		*lp++ ^= *rp++;
} 

// compares two arrays of 8-bit bytes and return true if the same or
// false otherwise

bool block_cmp(const u1byte l[], const u1byte r[], const u4byte len = 16)
{	const u1byte	*lp = l, *rp = r;

	while(lp < l + len)
	
		if(*lp++ != *rp++)
		
			return false;
	
	return true;
} 

// Extract decimal number from a string

int get_dec(std::string s)
{   std::string::const_iterator  p = s.begin();
    int		nbr = 0;

    while(p != s.end() && *p >= '0' && *p <= '9')
    {
        nbr = 10 * nbr + (*p - '0'); ++p;
    }

    return nbr;
}

// Find a line with a given header and return line type
// or -1 if end of file

int find_line(std::ifstream &inf, std::string &str)
{   std::string::size_type   pos;
    int                 ty1;

    while(!inf.eof())
    {
        std::getline(inf, str);				// input a line

        for(ty1 = 0; ty1 < no_lhdr; ++ty1)	// compare line header
        {
            if((pos = str.find(fstr[ty1])) != std::string::npos && !pos)
            {
                return ty1;		// exit if known line header found
            }
        }
    }

    return -1;	// end of file
}

// Compare two strings ignoring case

int cmp_nocase(const std::string &s1, const std::string &s2)
{   std::string::const_iterator  p1 = s1.begin();
    std::string::const_iterator  p2 = s2.begin();

    while(p1 != s1.end() && p2 != s2.end())
    {
        if(toupper(*p1) != toupper(*p2))
        {
            return (toupper(*p1) < toupper(*p2) ? -1 : 1);
        }

        ++p1; ++p2;
    }

    return s2.size() - s1.size();
}

// Input command line arguments

bool test_args(int argc, char *argv[], char des_chr, char tst_chr = '\0')
{
    for(int i = 1; i < argc; ++i)
    {
        if(argv[i][0] != '-' && argv[i][0] != '/')

            continue;

        if(argv[i][1] != tolower(des_chr) && argv[i][1] != toupper(des_chr))
		
			continue;

		else if(!tst_chr)

			return true;

        if(argv[i][2] != ':')

            continue;

        for(int j = 3; argv[i][j]; ++j)

			if(argv[i][j] == tolower(tst_chr) || argv[i][j] == toupper(tst_chr))

				return true;
    }

    return false;
}

